@@ -9,9 +9,8 @@ Created on Fri Apr 10 14:12:56 2020
 import sys
 import math
 import numpy as np
-
-import spatialmath.base as tr
-import spatialmath.base.argcheck as argcheck
+from spatialmath import base as tr
+from spatialmath.base import argcheck
 
 _eps = np.finfo(np.float64).eps
 
@@ -164,7 +163,7 @@ def v2q(v):
     .. seealso:: q2v
     """
     v = argcheck.getvector(v,3)
-    s = 1 - np.linalg.norm(v)
+    s = math.sqrt(1 - np.sum(v**2))
     return np.r_[s, v]
 
 def qqmul(q1, q2):
@@ -181,7 +180,7 @@ def qqmul(q1, q2):
     This is the quaternion or Hamilton product.  If both operands are unit-quaternions then
     the product will be a unit-quaternion.
     
-    :seealso: qvmul
+    :seealso: qvmul, inner
 
     """
     q1 = argcheck.getvector(q1,4)
@@ -191,6 +190,29 @@ def qqmul(q1, q2):
     
     return np.r_[s1 * s2 - np.dot(v1, v2), s1 * v2 + s2 * v1 + np.cross(v1, v2)]
 
+
+def inner(q1, q2):
+    """
+    Quaternion innert product
+    
+    :arg q0: quaternion as a 4-vector
+    :type q0: : array_like
+    :arg q1: uaternion as a 4-vector
+    :type q1: array_like
+    :return: inner product
+    :rtype: numpy.ndarray, shape=(4,)
+    
+    This is the inner or dot product of two quaternions, it is the sum of the element-wise
+    product.  
+    
+    :seealso: qvmul
+
+    """
+    q1 = argcheck.getvector(q1,4)
+    q2 = argcheck.getvector(q2,4)
+
+    return np.dot(q1, q2)
+
 def qvmul(q, v):
     """
     Vector rotation
@@ -214,6 +236,20 @@ def qvmul(q, v):
     qv = qqmul(q, qqmul(pure(v), conj(q)))
     return qv[1:4]
 
+def vvmul(qa, qb):
+    """
+            %UnitQuaternion.QVMUL Multiply unit quaternions defined by vector part
+        %
+        % QV = UnitQuaternion.QVMUL(QV1, QV2) multiplies two unit-quaternions
+        % defined only by their vector components QV1 and QV2 (3x1).  The result is
+        % similarly the vector component of the Hamilton product (3x1).
+        %
+        % Notes::
+        % - Is a static class meth
+    """
+    t6 = math.sqrt(1.0 - np.sum(qa**2));
+    t11 = math.sqrt(1.0 - np.sum(qb**2));
+    return np.r_[qa[1]*qb[2]-qb[1]*qa[2]+qb[0]*t6+qa[0]*t11, -qa[0]*qb[2]+qb[0]*qa[2]+qb[1]*t6+qa[1]*t11,qa[0]*qb[1]-qb[0]*qa[1]+qb[2]*t6+qa[2]*t11]
 
 def pow(q, power):
     """
@@ -334,7 +370,7 @@ def r2q(R, check=True):
     kv = np.r_[kx, ky, kz]
     nm = np.linalg.norm(kv)
     if abs(nm) <  100*_eps:
-        return qone()
+        return eye()
     else:
         return np.r_[qs, (math.sqrt(1.0 - qs ** 2) / nm) * kv]
     
@@ -536,8 +572,8 @@ def qprint(q, delim=('<', '>'), fmt='%f', file=sys.stdout):
     else:
         return s
     
-if __name__ == '__main__':
+if __name__ == '__main__':  # pragma: no cover
     import pathlib
     import os.path
     
-    runfile(os.path.join(pathlib.Path(__file__).parent.absolute(), "test_quaternion.py") )
+    exec(open(os.path.join(pathlib.Path(__file__).parent.absolute(), "test_quaternions.py")).read() )
\ No newline at end of file
