@@ -8,7 +8,7 @@ from math import pi
 from spatialmath.pose3d import *
 from spatialmath import super_pose as sp
 from spatialmath.base import *
-import spatialmath.base.argcheck as argcheck
+from spatialmath.base import argcheck
 
 def array_compare(x, y):
     if isinstance(x, sp.SMPose):
@@ -19,94 +19,94 @@ def array_compare(x, y):
                            
 
 class TestSO3(unittest.TestCase):
-    
+
     def test_constructor(self):
-        
+
         # null constructor
         R = SO3()
         nt.assert_equal(len(R),  1)
         array_compare(R, np.eye(3))
-        nt.assert_equal(isinstance(R, SO3), True)
-        
+        self.assertIsInstance(R, SO3)
+
         # construct from matrix
-        R = SO3( rotx(0.2) )
+        R = SO3(rotx(0.2))
         nt.assert_equal(len(R),  1)
         array_compare(R, rotx(0.2))
-        nt.assert_equal(isinstance(R, SO3), True)
-        
+        self.assertIsInstance(R, SO3)
+
         # construct from canonic rotation
         R = SO3.Rx(0.2)
         nt.assert_equal(len(R),  1)
         array_compare(R, rotx(0.2))
-        nt.assert_equal(isinstance(R, SO3), True)
-        
+        self.assertIsInstance(R, SO3)
+
         R = SO3.Ry(0.2)
         nt.assert_equal(len(R),  1)
         array_compare(R, roty(0.2))
-        nt.assert_equal(isinstance(R, SO3), True)
-        
+        self.assertIsInstance(R, SO3)
+
         R = SO3.Rz(0.2)
         nt.assert_equal(len(R),  1)
         array_compare(R, rotz(0.2))
-        nt.assert_equal(isinstance(R, SO3), True)
-        
+        self.assertIsInstance(R, SO3)
+
         # triple angle
         R = SO3.Eul([0.1, 0.2, 0.3])
         nt.assert_equal(len(R),  1)
         array_compare(R, eul2r([0.1, 0.2, 0.3]))
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         
         R = SO3.Eul(np.r_[0.1, 0.2, 0.3])
         nt.assert_equal(len(R),  1)
         array_compare(R, eul2r([0.1, 0.2, 0.3]))
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         
         R = SO3.Eul([10, 20, 30], unit='deg')
         nt.assert_equal(len(R),  1)
         array_compare(R, eul2r([10, 20, 30], unit='deg'))
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         
         R = SO3.RPY([0.1, 0.2, 0.3])
         nt.assert_equal(len(R),  1)
         array_compare(R, rpy2r([0.1, 0.2, 0.3]))
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         
         R = SO3.RPY(np.r_[0.1, 0.2, 0.3])
         nt.assert_equal(len(R),  1)
         array_compare(R, rpy2r([0.1, 0.2, 0.3]))
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         
         R = SO3.RPY([10, 20, 30], unit='deg')
         nt.assert_equal(len(R),  1)
         array_compare(R, rpy2r([10, 20, 30], unit='deg'))
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         
         R = SO3.RPY([0.1, 0.2, 0.3], order='xyz')
         nt.assert_equal(len(R),  1)
         array_compare(R, rpy2r([0.1, 0.2, 0.3], order='xyz'))
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
              
         # angvec
         R = SO3.AngVec(0.2, [1, 0, 0])
         nt.assert_equal(len(R),  1)
         array_compare(R, rotx(0.2))
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         
         R = SO3.AngVec(0.3, [0, 1, 0])
         nt.assert_equal(len(R),  1)
         array_compare(R, roty(0.3))
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         
         # OA
         R = SO3.OA([0, 1, 0], [0, 0, 1])
         nt.assert_equal(len(R),  1)
         array_compare(R, np.eye(3))
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         
         # random
         R = SO3.Rand()
         nt.assert_equal(len(R),  1)
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         
         # copy constructor
         R = SO3.Rx(pi/2)
@@ -123,7 +123,7 @@ class TestSO3(unittest.TestCase):
         R.append(R1)
         R.append(R2)
         nt.assert_equal(len(R),  3)
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         
         array_compare(R[0], np.eye(3))
         array_compare(R[1], R1)
@@ -131,14 +131,14 @@ class TestSO3(unittest.TestCase):
         
         R = SO3([rotx(0.1), rotx(0.2), rotx(0.3)])
         nt.assert_equal(len(R),  3)
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         array_compare(R[0], rotx(0.1))
         array_compare(R[1], rotx(0.2))
         array_compare(R[2], rotx(0.3))
         
         R = SO3([SO3.Rx(0.1), SO3.Rx(0.2), SO3.Rx(0.3)])
         nt.assert_equal(len(R),  3)
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         array_compare(R[0], rotx(0.1))
         array_compare(R[1], rotx(0.2))
         array_compare(R[2], rotx(0.3))
@@ -169,97 +169,97 @@ class TestSO3(unittest.TestCase):
         nt.assert_equal(R.shape, (3,3))
         
         R = SO3.Rx(0.3)
-        array_compare(R.T * R, np.eye(3,3))
+        array_compare(R.inv * R, np.eye(3,3))
         
     def test_arith(self):
         R = SO3()
         
         # sum
         a = R + R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         array_compare(a, np.array([ [2,0,0], [0,2,0], [0,0,2]]))
         
         a = R + 1
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         array_compare(a, np.array([ [2,1,1], [1,2,1], [1,1,2]]))
         
         # a = 1 + R
-        # nt.assert_equal(isinstance(a, SO3), False)
+        # self.assertNotIsInstance(a, SO3)
         # array_compare(a, np.array([ [2,1,1], [1,2,1], [1,1,2]]))
         
         a = R + np.eye(3)
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         array_compare(a, np.array([ [2,0,0], [0,2,0], [0,0,2]]))
         
         # a =  np.eye(3) + R
-        # nt.assert_equal(isinstance(a, SO3), False)
+        # self.assertNotIsInstance(a, SO3)
         # array_compare(a, np.array([ [2,0,0], [0,2,0], [0,0,2]]))
         #  this invokes the __add__ method for numpy
         
         R += R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         array_compare(a, np.array([ [2,0,0], [0,2,0], [0,0,2]]))
         
         R = SO3()
         R += 1
-        nt.assert_equal(isinstance(R, SO3), False)
+        self.assertNotIsInstance(R, SO3)
         array_compare(R, np.array([ [2,1,1], [1,2,1], [1,1,2]]))
         
         # difference
         R = SO3()
          
         a = R - R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         array_compare(a, np.zeros((3,3)))
         
         a = R - 1
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         array_compare(a, np.array([ [0,-1,-1], [-1,0,-1], [-1,-1,0]]))
         
         # a = 1 - R
-        # nt.assert_equal(isinstance(a, SO3), False)
+        # self.assertNotIsInstance(a, SO3)
         # array_compare(a, -np.array([ [0,-1,-1], [-1,0,-1], [-1,-1,0]]))
         
         a = R - np.eye(3)
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         array_compare(a, np.zeros((3,3)))
 
         # a =  np.eye(3) - R
-        # nt.assert_equal(isinstance(a, SO3), False)
+        # self.assertNotIsInstance(a, SO3)
         # array_compare(a, np.zeros((3,3)))
         
         R -= R
-        nt.assert_equal(isinstance(R, SO3), False)
+        self.assertNotIsInstance(R, SO3)
         array_compare(R, np.zeros((3,3)))
         
         R = SO3()
         R -= 1
-        nt.assert_equal(isinstance(R, SO3), False)
+        self.assertNotIsInstance(R, SO3)
         array_compare(R, np.array([ [0,-1,-1], [-1,0,-1], [-1,-1,0]]))
 
         # multiply
         R = SO3()
         
         a = R * R
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         array_compare(a, R)
         
         a = R * 2
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         array_compare(a, 2*np.eye(3))
         
         a = 2 * R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         array_compare(a, 2*np.eye(3))
         
         R = SO3()
         R *= SO3.Rx(pi/2)
-        nt.assert_equal(isinstance(R, SO3), True)
+        self.assertIsInstance(R, SO3)
         array_compare(R, rotx(pi/2))
         
         R = SO3()
         R *= 2
-        nt.assert_equal(isinstance(R, SO3), False)
+        self.assertNotIsInstance(R, SO3)
         array_compare(R, 2*np.eye(3))
         
         array_compare(SO3.Rx(pi/2) * SO3.Ry(pi/2) * SO3.Rx(-pi/2), SO3.Rz(pi/2))
@@ -290,11 +290,11 @@ class TestSO3(unittest.TestCase):
         # divide
         R = SO3.Ry(0.3)
         a = R / R
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         array_compare(a, np.eye(3))
         
         a = R / 2
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         array_compare(a, roty(0.3)/2)
 
 
@@ -309,35 +309,35 @@ class TestSO3(unittest.TestCase):
         # multiply
         R = SO3([rx, ry, rz])
         a = R * rx
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], ry*rx)
         array_compare(a[2], rz*rx)
         
         a = rx * R
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], rx*ry)
         array_compare(a[2], rx*rz)
         
         a = R * R
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], ry*ry)
         array_compare(a[2], rz*rz)
                        
         a = R * 2
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*2)
         array_compare(a[1], ry*2)
         array_compare(a[2], rz*2)
         
         a = 2 * R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*2)
         array_compare(a[1], ry*2)
@@ -345,7 +345,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a *= rx
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], ry*rx)
@@ -353,7 +353,7 @@ class TestSO3(unittest.TestCase):
         
         a = rx
         a *= R
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], rx*ry)
@@ -361,7 +361,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a *= R
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], ry*ry)
@@ -369,7 +369,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a *= 2
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*2)
         array_compare(a[1], ry*2)
@@ -394,28 +394,28 @@ class TestSO3(unittest.TestCase):
         # divide
         R = SO3([rx, ry, rz])
         a = R / rx
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/rx)
         array_compare(a[1], ry/rx)
         array_compare(a[2], rz/rx)
         
         a = rx / R
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/rx)
         array_compare(a[1], rx/ry)
         array_compare(a[2], rx/rz)
         
         a = R / R
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], np.eye(3))
         array_compare(a[1], np.eye(3))
         array_compare(a[2], np.eye(3))
         
         a = R / 2
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/2)
         array_compare(a[1], ry/2)
@@ -423,7 +423,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a /= rx
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/rx)
         array_compare(a[1], ry/rx)
@@ -431,7 +431,7 @@ class TestSO3(unittest.TestCase):
         
         a = rx
         a /= R
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/rx)
         array_compare(a[1], rx/ry)
@@ -439,7 +439,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a /= R
-        nt.assert_equal(isinstance(a, SO3), True)
+        self.assertIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], np.eye(3))
         array_compare(a[1], np.eye(3))
@@ -447,7 +447,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a /= 2
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/2)
         array_compare(a[1], ry/2)
@@ -456,28 +456,28 @@ class TestSO3(unittest.TestCase):
         # add
         R = SO3([rx, ry, rz])
         a = R + rx
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], ry+rx)
         array_compare(a[2], rz+rx)
         
         a = rx + R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], rx+ry)
         array_compare(a[2], rx+rz)
         
         a = R + R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], ry+ry)
         array_compare(a[2], rz+rz)
         
         a = R + 1
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+1)
         array_compare(a[1], ry+1)
@@ -485,7 +485,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a += rx
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], ry+rx)
@@ -493,7 +493,7 @@ class TestSO3(unittest.TestCase):
         
         a = rx
         a += R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], rx+ry)
@@ -501,7 +501,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a += R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], ry+ry)
@@ -509,7 +509,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a += 1
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+1)
         array_compare(a[1], ry+1)
@@ -518,28 +518,28 @@ class TestSO3(unittest.TestCase):
         # subtract
         R = SO3([rx, ry, rz])
         a = R - rx
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], ry-rx)
         array_compare(a[2], rz-rx)
         
         a = rx - R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], rx-ry)
         array_compare(a[2], rx-rz)
         
         a = R - R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], ry-ry)
         array_compare(a[2], rz-rz)
         
         a = R - 1
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-1)
         array_compare(a[1], ry-1)
@@ -547,7 +547,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a -= rx
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], ry-rx)
@@ -555,7 +555,7 @@ class TestSO3(unittest.TestCase):
         
         a = rx
         a -= R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], rx-ry)
@@ -563,7 +563,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a -= R
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], ry-ry)
@@ -571,7 +571,7 @@ class TestSO3(unittest.TestCase):
         
         a = R
         a -= 1
-        nt.assert_equal(isinstance(a, SO3), False)
+        self.assertNotIsInstance(a, SO3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-1)
         array_compare(a[1], ry-1)
@@ -597,104 +597,104 @@ class TestSE3(unittest.TestCase):
         R = SE3()
         nt.assert_equal(len(R),  1)
         array_compare(R, np.eye(4))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         # construct from matrix
         R = SE3( trotx(0.2) )
         nt.assert_equal(len(R),  1)
         array_compare(R, trotx(0.2))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         # construct from canonic rotation
         R = SE3.Rx(0.2)
         nt.assert_equal(len(R),  1)
         array_compare(R, trotx(0.2))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         R = SE3.Ry(0.2)
         nt.assert_equal(len(R),  1)
         array_compare(R, troty(0.2))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         R = SE3.Rz(0.2)
         nt.assert_equal(len(R),  1)
         array_compare(R, trotz(0.2))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         # construct from canonic translation
         R = SE3.Tx(0.2)
         nt.assert_equal(len(R),  1)
         array_compare(R, transl(0.2, 0, 0))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         R = SE3.Ty(0.2)
         nt.assert_equal(len(R),  1)
         array_compare(R, transl(0, 0.2, 0))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         R = SE3.Tz(0.2)
         nt.assert_equal(len(R),  1)
         array_compare(R, transl(0, 0, 0.2))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         
         # triple angle
         R = SE3.Eul([0.1, 0.2, 0.3])
         nt.assert_equal(len(R),  1)
         array_compare(R, eul2tr([0.1, 0.2, 0.3]))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         R = SE3.Eul(np.r_[0.1, 0.2, 0.3])
         nt.assert_equal(len(R),  1)
         array_compare(R, eul2tr([0.1, 0.2, 0.3]))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         R = SE3.Eul([10, 20, 30], unit='deg')
         nt.assert_equal(len(R),  1)
         array_compare(R, eul2tr([10, 20, 30], unit='deg'))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         R = SE3.RPY([0.1, 0.2, 0.3])
         nt.assert_equal(len(R),  1)
         array_compare(R, rpy2tr([0.1, 0.2, 0.3]))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         R = SE3.RPY(np.r_[0.1, 0.2, 0.3])
         nt.assert_equal(len(R),  1)
         array_compare(R, rpy2tr([0.1, 0.2, 0.3]))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         R = SE3.RPY([10, 20, 30], unit='deg')
         nt.assert_equal(len(R),  1)
         array_compare(R, rpy2tr([10, 20, 30], unit='deg'))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         R = SE3.RPY([0.1, 0.2, 0.3], order='xyz')
         nt.assert_equal(len(R),  1)
         array_compare(R, rpy2tr([0.1, 0.2, 0.3], order='xyz'))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
              
         # angvec
         R = SE3.AngVec(0.2, [1, 0, 0])
         nt.assert_equal(len(R),  1)
         array_compare(R, trotx(0.2))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         R = SE3.AngVec(0.3, [0, 1, 0])
         nt.assert_equal(len(R),  1)
         array_compare(R, troty(0.3))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         # OA
         R = SE3.OA([0, 1, 0], [0, 0, 1])
         nt.assert_equal(len(R),  1)
         array_compare(R, np.eye(4))
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         # random
         R = SE3.Rand()
         nt.assert_equal(len(R),  1)
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         # copy constructor
         R = SE3.Rx(pi/2)
@@ -711,7 +711,7 @@ class TestSE3(unittest.TestCase):
         R.append(R1)
         R.append(R2)
         nt.assert_equal(len(R),  3)
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         
         array_compare(R[0], np.eye(4))
         array_compare(R[1], R1)
@@ -719,14 +719,14 @@ class TestSE3(unittest.TestCase):
         
         R = SE3([trotx(0.1), trotx(0.2), trotx(0.3)])
         nt.assert_equal(len(R),  3)
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         array_compare(R[0], trotx(0.1))
         array_compare(R[1], trotx(0.2))
         array_compare(R[2], trotx(0.3))
         
         R = SE3([SE3.Rx(0.1), SE3.Rx(0.2), SE3.Rx(0.3)])
         nt.assert_equal(len(R),  3)
-        nt.assert_equal(isinstance(R, SE3), True)
+        self.assertIsInstance(R, SE3)
         array_compare(R[0], trotx(0.1))
         array_compare(R[1], trotx(0.2))
         array_compare(R[2], trotx(0.3))
@@ -762,93 +762,93 @@ class TestSE3(unittest.TestCase):
         
         # sum
         a = T + T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, np.array([ [2,0,0,2], [0,2,0,4], [0,0,2,6], [0,0,0,2]]))
 
         
         a = T + 1
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, np.array([ [2,1,1,2], [1,2,1,3], [1,1,2,4], [1,1,1,2]]))
         
         # a = 1 + T
-        # nt.assert_equal(isinstance(a, SE3), False)
+        # self.assertNotIsInstance(a, SE3)
         # array_compare(a, np.array([ [2,1,1], [1,2,1], [1,1,2]]))
         
         a = T + np.eye(4)
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, np.array([ [2,0,0,1], [0,2,0,2], [0,0,2,3], [0,0,0,2]]))
         
         # a =  np.eye(3) + T
-        # nt.assert_equal(isinstance(a, SE3), False)
+        # self.assertNotIsInstance(a, SE3)
         # array_compare(a, np.array([ [2,0,0], [0,2,0], [0,0,2]]))
         #  this invokes the __add__ method for numpy
         
         a = T
         a += T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, np.array([ [2,0,0,2], [0,2,0,4], [0,0,2,6], [0,0,0,2]]))
         
         a = T
         a += 1
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, np.array([ [2,1,1,2], [1,2,1,3], [1,1,2,4], [1,1,1,2]]))
         
         # difference
         T = SE3.trans(1,2,3)
          
         a = T - T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, np.zeros((4,4)))
         
         a = T - 1
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, np.array([ [0,-1,-1,0], [-1,0,-1,1], [-1,-1,0,2], [-1,-1,-1,0]]))
         
         # a = 1 - T
-        # nt.assert_equal(isinstance(a, SE3), False)
+        # self.assertNotIsInstance(a, SE3)
         # array_compare(a, -np.array([ [0,-1,-1], [-1,0,-1], [-1,-1,0]]))
         
         a = T - np.eye(4)
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, np.array([[0,0,0,1],[0,0,0,2],[0,0,0,3],[0,0,0,0]]))
 
         # a =  np.eye(3) - T
-        # nt.assert_equal(isinstance(a, SE3), False)
+        # self.assertNotIsInstance(a, SE3)
         # array_compare(a, np.zeros((3,3)))
         
         a = T
         a -= T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, np.zeros((4,4)))
         
         a = T
         a -= 1
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, np.array([ [0,-1,-1,0], [-1,0,-1,1], [-1,-1,0,2], [-1,-1,-1,0]]))
 
         # multiply
         T = SE3.trans(1,2,3)
         
         a = T * T
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         array_compare(a, transl(2,4,6))
         
         a = T * 2
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, 2*transl(1,2,3))
         
         a = 2 * T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, 2*transl(1,2,3))
         
         T = SE3.trans(1,2,3)
         T *= SE3.Ry(pi/2)
-        nt.assert_equal(isinstance(T, SE3), True)
+        self.assertIsInstance(T, SE3)
         array_compare(T, np.array([[0,0,1,1],[0,1,0,2],[-1,0,0,3],[0,0,0,1]]))
         
         T = SE3()
         T *= 2
-        nt.assert_equal(isinstance(T, SE3), False)
+        self.assertNotIsInstance(T, SE3)
         array_compare(T, 2*np.eye(4))
         
         array_compare(SE3.Rx(pi/2) * SE3.Ry(pi/2) * SE3.Rx(-pi/2), SE3.Rz(pi/2))
@@ -879,11 +879,11 @@ class TestSE3(unittest.TestCase):
         # divide
         T = SE3.Ry(0.3)
         a = T / T
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         array_compare(a, np.eye(4))
         
         a = T / 2
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         array_compare(a, troty(0.3)/2)
 
 
@@ -898,35 +898,35 @@ class TestSE3(unittest.TestCase):
         # multiply
         T = SE3([rx, ry, rz])
         a = T * rx
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], ry*rx)
         array_compare(a[2], rz*rx)
         
         a = rx * T
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], rx*ry)
         array_compare(a[2], rx*rz)
         
         a = T * T
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], ry*ry)
         array_compare(a[2], rz*rz)
                        
         a = T * 2
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*2)
         array_compare(a[1], ry*2)
         array_compare(a[2], rz*2)
         
         a = 2 * T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*2)
         array_compare(a[1], ry*2)
@@ -934,7 +934,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a *= rx
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], ry*rx)
@@ -942,7 +942,7 @@ class TestSE3(unittest.TestCase):
         
         a = rx
         a *= T
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], rx*ry)
@@ -950,7 +950,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a *= T
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*rx)
         array_compare(a[1], ry*ry)
@@ -958,7 +958,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a *= 2
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx*2)
         array_compare(a[1], ry*2)
@@ -983,28 +983,28 @@ class TestSE3(unittest.TestCase):
         # divide
         T = SE3([rx, ry, rz])
         a = T / rx
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/rx)
         array_compare(a[1], ry/rx)
         array_compare(a[2], rz/rx)
         
         a = rx / T
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/rx)
         array_compare(a[1], rx/ry)
         array_compare(a[2], rx/rz)
         
         a = T / T
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], np.eye(4))
         array_compare(a[1], np.eye(4))
         array_compare(a[2], np.eye(4))
         
         a = T / 2
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/2)
         array_compare(a[1], ry/2)
@@ -1012,7 +1012,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a /= rx
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/rx)
         array_compare(a[1], ry/rx)
@@ -1020,7 +1020,7 @@ class TestSE3(unittest.TestCase):
         
         a = rx
         a /= T
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/rx)
         array_compare(a[1], rx/ry)
@@ -1028,7 +1028,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a /= T
-        nt.assert_equal(isinstance(a, SE3), True)
+        self.assertIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], np.eye(4))
         array_compare(a[1], np.eye(4))
@@ -1036,7 +1036,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a /= 2
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx/2)
         array_compare(a[1], ry/2)
@@ -1045,28 +1045,28 @@ class TestSE3(unittest.TestCase):
         # add
         T = SE3([rx, ry, rz])
         a = T + rx
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], ry+rx)
         array_compare(a[2], rz+rx)
         
         a = rx + T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], rx+ry)
         array_compare(a[2], rx+rz)
         
         a = T + T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], ry+ry)
         array_compare(a[2], rz+rz)
         
         a = T + 1
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+1)
         array_compare(a[1], ry+1)
@@ -1074,7 +1074,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a += rx
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], ry+rx)
@@ -1082,7 +1082,7 @@ class TestSE3(unittest.TestCase):
         
         a = rx
         a += T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], rx+ry)
@@ -1090,7 +1090,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a += T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+rx)
         array_compare(a[1], ry+ry)
@@ -1098,7 +1098,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a += 1
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx+1)
         array_compare(a[1], ry+1)
@@ -1107,28 +1107,28 @@ class TestSE3(unittest.TestCase):
         # subtract
         T = SE3([rx, ry, rz])
         a = T - rx
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], ry-rx)
         array_compare(a[2], rz-rx)
         
         a = rx - T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], rx-ry)
         array_compare(a[2], rx-rz)
         
         a = T - T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], ry-ry)
         array_compare(a[2], rz-rz)
         
         a = T - 1
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-1)
         array_compare(a[1], ry-1)
@@ -1136,7 +1136,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a -= rx
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], ry-rx)
@@ -1144,7 +1144,7 @@ class TestSE3(unittest.TestCase):
         
         a = rx
         a -= T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], rx-ry)
@@ -1152,7 +1152,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a -= T
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-rx)
         array_compare(a[1], ry-ry)
@@ -1160,7 +1160,7 @@ class TestSE3(unittest.TestCase):
         
         a = T
         a -= 1
-        nt.assert_equal(isinstance(a, SE3), False)
+        self.assertNotIsInstance(a, SE3)
         nt.assert_equal(len(a), 3)
         array_compare(a[0], rx-1)
         array_compare(a[1], ry-1)
